@@ -1,6 +1,6 @@
 use indicatif::{ProgressBar, ProgressFinish, ProgressStyle};
 use serde::Serialize;
-use std::{io, num::TryFromIntError, path::Path};
+use std::{io, num::TryFromIntError, path::Path, time::Duration};
 
 use git2::{
     build::{CheckoutBuilder, RepoBuilder},
@@ -42,13 +42,14 @@ pub fn sync_crates_repo(mirror_path: &Path, crates: &ConfigCrates) -> Result<(),
         .with_style(
             ProgressStyle::default_bar()
                 .template("{prefix} {wide_bar} {spinner} [{elapsed_precise}]")
-                .progress_chars("  ")
-                .on_finish(ProgressFinish::AndLeave),
+                .expect("template is correct")
+                .progress_chars("  "),
         )
+        .with_finish(ProgressFinish::AndLeave)
         .with_prefix(prefix);
     // Enable the steady tick, so the transfer progress callback isn't spending its time
     // updating the progress bar.
-    pb.enable_steady_tick(10);
+    pb.enable_steady_tick(Duration::from_millis(10));
 
     // Libgit2 has callbacks that allow us to update the progress bar
     // as the git download progresses.
@@ -76,17 +77,15 @@ pub fn sync_crates_repo(mirror_path: &Path, crates: &ConfigCrates) -> Result<(),
     fetch_opts.proxy_options(proxy_opts);
 
     if !repo_path.join(".git").exists() {
-        clone_repository(fetch_opts, &crates.source_index, &repo_path)?
+        clone_repository(fetch_opts, &crates.source_index, &repo_path)?;
+        // Remove master in order to ensure full scan is performed
+        let repo = Repository::open(&repo_path)?;
+        repo.find_reference("refs/heads/master")?.delete()?;
     } else {
         // Get (fetch) the branch's latest remote "master" commit
         let repo = Repository::open(&repo_path)?;
         let mut remote = repo.find_remote("origin")?;
         remote.fetch(&["master"], Some(&mut fetch_opts), None)?;
-
-        // Set master to origin/master.
-        //
-        // Note that this means config.json changes will have to be rewritten on every sync.
-        fast_forward(&repo_path)?;
     }
 
     Ok(())
@@ -108,7 +107,7 @@ pub fn update_crates_config(
 
 /// Perform a git fast-forward on the repository. This will destroy any local changes that have
 /// been made to the repo, and will make the local master identical to the remote master.
-fn fast_forward(repo_path: &Path) -> Result<(), IndexSyncError> {
+pub fn fast_forward(repo_path: &Path) -> Result<(), IndexSyncError> {
     let repo = Repository::open(repo_path)?;
 
     let fetch_head = repo.find_reference("refs/remotes/origin/master")?;
@@ -159,7 +158,10 @@ pub fn rewrite_config_json(repo_path: &Path, base_url: &str) -> Result<(), Index
 
     let mut index = repo.index()?;
 
-    let crate_path = format!("{}/{}", base_url, "{crate}/{crate}-{version}.crate");
+    let crate_path = format!(
+        "{}/{}",
+        base_url, "{prefix}/{crate}/{version}/{crate}-{version}.crate"
+    );
 
     // Create the new config.json.
     let config_json = ConfigJson {
