@@ -11,8 +11,10 @@ use reqwest::header::HeaderValue;
 use serde::{Deserialize, Serialize};
 use std::collections::{HashMap, HashSet};
 use std::path::{Path, PathBuf};
+use std::time::Duration;
 use std::{fs, io};
 use thiserror::Error;
+use tokio::task::JoinError;
 
 // The allowed platforms to validate the configuration
 // Note: These platforms should match the list on https://rust-lang.github.io/rustup/installation/other.html
@@ -104,6 +106,7 @@ static PLATFORMS_WINDOWS: &[&str] = &[
     "x86_64-pc-windows-gnu",
     "x86_64-pc-windows-msvc",
 ];
+
 #[derive(Error, Debug)]
 pub enum SyncError {
     #[error("IO error: {0}")]
@@ -152,8 +155,6 @@ pub struct Channel {
 
 #[derive(Deserialize, Debug)]
 struct Release {
-    #[serde(alias = "schema-version")]
-    schema_version: String,
     version: String,
 }
 
@@ -163,37 +164,28 @@ pub struct Platforms {
     windows: Vec<String>,
 }
 
+impl Platforms {
+    // &String instead of &str is required due to vec.contains not performing proper inference
+    // here. See:
+    // https://stackoverflow.com/questions/48985924/why-does-a-str-not-coerce-to-a-string-when-using-veccontains
+    // https://github.com/rust-lang/rust/issues/42671
+    #[allow(clippy::ptr_arg)]
+    pub fn contains(&self, platform: &String) -> bool {
+        self.unix.contains(platform) || self.windows.contains(platform)
+    }
+
+    pub fn len(&self) -> usize {
+        self.unix.len() + self.windows.len()
+    }
+}
+
 pub fn get_platforms(rustup: &ConfigRustup) -> Result<Platforms, MirrorError> {
     let unix = match &rustup.platforms_unix {
-        Some(p) => {
-            let bad_platforms: Vec<&String> = p
-                .iter()
-                .filter(|x| !PLATFORMS_UNIX.contains(&x.as_str()))
-                .collect();
-            if !bad_platforms.is_empty() {
-                eprintln!("Bad values in unix platforms: {:?}", bad_platforms);
-                return Err(MirrorError::Config(
-                    "bad value for 'platforms_unix'".to_string(),
-                ));
-            }
-            p.clone()
-        }
+        Some(p) => p.clone(),
         None => PLATFORMS_UNIX.iter().map(|x| x.to_string()).collect(),
     };
     let windows = match &rustup.platforms_windows {
-        Some(p) => {
-            let bad_platforms: Vec<&String> = p
-                .iter()
-                .filter(|x| !PLATFORMS_WINDOWS.contains(&x.as_str()))
-                .collect();
-            if !bad_platforms.is_empty() {
-                eprintln!("Bad values in windows platforms: {:?}", bad_platforms);
-                return Err(MirrorError::Config(
-                    "bad value for 'platforms_windows'".to_string(),
-                ));
-            }
-            p.clone()
-        }
+        Some(p) => p.clone(),
         None => PLATFORMS_WINDOWS.iter().map(|x| x.to_string()).collect(),
     };
     Ok(Platforms { unix, windows })
@@ -238,6 +230,63 @@ pub async fn sync_one_init(
     Ok(())
 }
 
+fn panamax_progress_bar(size: usize, prefix: String) -> ProgressBar {
+    ProgressBar::new(size as u64)
+        .with_style(
+            ProgressStyle::default_bar()
+                .template(
+                    "{prefix} {wide_bar} {pos}/{len} [{elapsed_precise} / {duration_precise}]",
+                )
+                .expect("template is correct")
+                .progress_chars("█▉▊▋▌▍▎▏  "),
+        )
+        .with_finish(ProgressFinish::AndLeave)
+        .with_prefix(prefix)
+}
+
+#[allow(clippy::too_many_arguments)]
+async fn create_sync_tasks(
+    platforms: &[String],
+    is_exe: bool,
+    rustup_version: &str,
+    path: &Path,
+    source: &str,
+    retries: usize,
+    user_agent: &HeaderValue,
+    threads: usize,
+    pb: &ProgressBar,
+) -> Vec<Result<Result<(), DownloadError>, JoinError>> {
+    futures::stream::iter(platforms.iter())
+        .map(|platform| {
+            let rustup_version = rustup_version.to_string();
+            let path = path.to_path_buf();
+            let source = source.to_string();
+            let user_agent = user_agent.clone();
+            let platform = platform.clone();
+            let pb = pb.clone();
+
+            tokio::spawn(async move {
+                let out = sync_one_init(
+                    &path,
+                    &source,
+                    platform.as_str(),
+                    is_exe,
+                    &rustup_version,
+                    retries,
+                    &user_agent,
+                )
+                .await;
+
+                pb.inc(1);
+
+                out
+            })
+        })
+        .buffer_unordered(threads)
+        .collect::<Vec<Result<_, _>>>()
+        .await
+}
+
 /// Synchronize all rustup-init files.
 pub async fn sync_rustup_init(
     path: &Path,
@@ -248,8 +297,6 @@ pub async fn sync_rustup_init(
     user_agent: &HeaderValue,
     platforms: &Platforms,
 ) -> Result<(), SyncError> {
-    let count = platforms.unix.len() + platforms.windows.len();
-
     let mut errors_occurred = 0usize;
 
     // Download rustup release file
@@ -271,48 +318,36 @@ pub async fn sync_rustup_init(
 
     move_if_exists(&release_part_path, &release_path)?;
 
-    let pb = ProgressBar::new(count as u64)
-        .with_style(
-            ProgressStyle::default_bar()
-                .template(
-                    "{prefix} {wide_bar} {pos}/{len} [{elapsed_precise} / {duration_precise}]",
-                )
-                .progress_chars("█▉▊▋▌▍▎▏  ")
-                .on_finish(ProgressFinish::AndLeave),
-        )
-        .with_prefix(prefix);
-    pb.enable_steady_tick(10);
+    let pb = panamax_progress_bar(platforms.len(), prefix);
+    pb.enable_steady_tick(Duration::from_millis(10));
 
-    let tasks = futures::stream::iter(platforms.unix.iter().chain(platforms.windows.iter()))
-        .map(|platform| {
-            // Clone the variables that will be moved into the tokio task.
-            let rustup_version = rustup_version.clone();
-            let path = path.to_path_buf();
-            let source = source.to_string();
-            let user_agent = user_agent.clone();
-            let platform = platform.clone();
-            let pb = pb.clone();
+    let unix_tasks = create_sync_tasks(
+        &platforms.unix,
+        false,
+        &rustup_version,
+        path,
+        source,
+        retries,
+        user_agent,
+        threads,
+        &pb,
+    )
+    .await;
 
-            tokio::spawn(async move {
-                pb.inc(1);
+    let win_tasks = create_sync_tasks(
+        &platforms.windows,
+        true,
+        &rustup_version,
+        path,
+        source,
+        retries,
+        user_agent,
+        threads,
+        &pb,
+    )
+    .await;
 
-                sync_one_init(
-                    &path,
-                    &source,
-                    platform.as_str(),
-                    false,
-                    &rustup_version,
-                    retries,
-                    &user_agent,
-                )
-                .await
-            })
-        })
-        .buffer_unordered(threads)
-        .collect::<Vec<_>>()
-        .await;
-
-    for res in tasks {
+    for res in unix_tasks.into_iter().chain(win_tasks) {
         // Unwrap the join result.
         let res = res.unwrap();
 
@@ -356,11 +391,9 @@ pub fn rustup_download_list(
             .flat_map(|(_, pkg)| {
                 pkg.target
                     .into_iter()
-                    .filter(|(name, _)| {
-                        platforms.unix.contains(name)
-                            || platforms.windows.contains(name)
-                            || name == "*" // The * platform contains rust-src, always download
-                    })
+                    .filter(
+                        |(name, _)| platforms.contains(name) || name == "*", // The * platform contains rust-src, always download
+                    )
                     .flat_map(|(_, target)| -> Vec<(String, String)> {
                         target
                             .target_urls
@@ -506,16 +539,7 @@ pub fn clean_old_files(
     }
 
     // Progress bar!
-    let pb = ProgressBar::new(files_to_delete.len() as u64)
-        .with_style(
-            ProgressStyle::default_bar()
-                .template(
-                    "{prefix} {wide_bar} {pos}/{len} [{elapsed_precise} / {duration_precise}]",
-                )
-                .progress_chars("█▉▊▋▌▍▎▏  ")
-                .on_finish(ProgressFinish::AndLeave),
-        )
-        .with_prefix(prefix);
+    let pb = panamax_progress_bar(files_to_delete.len(), prefix);
 
     for f in files_to_delete {
         if let Err(e) = fs::remove_file(path.join(&f)) {
@@ -538,6 +562,7 @@ pub fn add_to_channel_history(
     channel: &str,
     date: &str,
     files: &[(String, String)],
+    extra_files: &[String],
 ) -> Result<(), SyncError> {
     let mut channel_history = match get_channel_history(path, channel) {
         Ok(c) => c,
@@ -547,10 +572,12 @@ pub fn add_to_channel_history(
         Err(e) => return Err(e),
     };
 
-    channel_history.versions.insert(
-        date.to_string(),
-        files.iter().map(|(f, _)| f.to_string()).collect(),
-    );
+    let files = files.iter().map(|(f, _)| f.to_string());
+    let extra_files = extra_files.iter().map(|ef| ef.to_string());
+
+    let files = files.chain(extra_files).collect();
+
+    channel_history.versions.insert(date.to_string(), files);
 
     let ch_data = toml::to_string(&channel_history)?;
 
@@ -582,8 +609,22 @@ pub async fn sync_rustup_channel(
     platforms: &Platforms,
 ) -> Result<(), SyncError> {
     // Download channel file
-    let channel_url = format!("{}/dist/channel-rust-{}.toml", source, channel);
-    let channel_path = path.join(format!("dist/channel-rust-{}.toml", channel));
+    let (channel_url, channel_path, extra_files) =
+        if let Some(inner_channel) = channel.strip_prefix("nightly-") {
+            let url = format!(
+                "{}/dist/{}/channel-rust-nightly.toml",
+                source, inner_channel
+            );
+            let path_chunk = format!("dist/{}/channel-rust-nightly.toml", inner_channel);
+            let path = path.join(&path_chunk);
+            // Make sure the cleanup step doesn't delete the channel toml
+            let extra_files = vec![path_chunk.clone(), format!("{}.sha256", path_chunk)];
+            (url, path, extra_files)
+        } else {
+            let url = format!("{}/dist/channel-rust-{}.toml", source, channel);
+            let path = path.join(format!("dist/channel-rust-{}.toml", channel));
+            (url, path, Vec::new())
+        };
     let channel_part_path = append_to_path(&channel_path, ".part");
     download_with_sha256_file(&channel_url, &channel_part_path, retries, true, user_agent).await?;
 
@@ -597,17 +638,8 @@ pub async fn sync_rustup_channel(
     )?;
     move_if_exists_with_sha256(&channel_part_path, &channel_path)?;
 
-    let pb = ProgressBar::new((files.len()) as u64)
-        .with_style(
-            ProgressStyle::default_bar()
-                .template(
-                    "{prefix} {wide_bar} {pos}/{len} [{elapsed_precise} / {duration_precise}]",
-                )
-                .progress_chars("█▉▊▋▌▍▎▏  ")
-                .on_finish(ProgressFinish::AndLeave),
-        )
-        .with_prefix(prefix);
-    pb.enable_steady_tick(10);
+    let pb = panamax_progress_bar(files.len(), prefix);
+    pb.enable_steady_tick(Duration::from_millis(10));
 
     let mut errors_occurred = 0usize;
 
@@ -622,9 +654,12 @@ pub async fn sync_rustup_channel(
             let pb = pb.clone();
 
             tokio::spawn(async move {
+                let out =
+                    sync_one_rustup_target(&path, &source, &url, &hash, retries, &user_agent).await;
+
                 pb.inc(1);
 
-                sync_one_rustup_target(&path, &source, &url, &hash, retries, &user_agent).await
+                out
             })
         })
         .buffer_unordered(threads)
@@ -648,7 +683,7 @@ pub async fn sync_rustup_channel(
 
     if errors_occurred == 0 {
         // Write channel history file
-        add_to_channel_history(path, channel, &date, &files)?;
+        add_to_channel_history(path, channel, &date, &files, &extra_files)?;
         Ok(())
     } else {
         Err(SyncError::FailedDownloads {
